#include <stdio.h>
#include <stdlib.h>
#include <string.h>

extern char **environ;

typedef struct {
  char name[256];
  char value[1024];
} query_pair;

unsigned parse_query_string(char * query, query_pair pairs[])
{
  /* Enum promenljiva koja odredjuje stanje u toku citanja para
     ime=vrednost */
  enum {    
    RD_NAME, /* u toku je citanje imena */
    RD_VALUE, /* u toku je citanje vrednosti */
    FINISHED /* zavrseno je citanje tekuceg para */
  } read_state = FINISHED;

  /* odredjujemo duzinu upitnog stringa */
  unsigned query_length = strlen(query); 
  int i = 0;
  int j = 0;
  int k = 0;

  /* Citamo karakter po karakter upitnog stringa */
  while(i < query_length)
    {
      /* Ako se nalazimo u stanju FINISHED, prvi sledeci karakter je
	 pocetak imena u novom paru */
      if(read_state == FINISHED)
	{
	  /* Prelazimo u stanje RD_NAME */
	  read_state = RD_NAME;
	  continue;
	}

      /* Ako se nalazimo usred imena u paru ime=vrednost */
      if(read_state == RD_NAME)
	{
	  /* Ako naidjemo na =, tada zatvaramo odgovarajuci string 
	     sa imenom i prelazimo na vrednost */
	  if(query[i] != '=')
	    {
	      pairs[j].name[k++] = query[i];
	    }
	  else
	    {
	      pairs[j].name[k] = '\0';
	      read_state = RD_VALUE;
	      k = 0;
	    }
	}
      /* Ako se nalazimo usred vrednosti u paru ime=vrednost */
      else
	{
	  /* Ako naidjemo na &, tada zatvaramo odgovarajuci string
	     sa vrednoscu i prelazimo u stanje FINISHED */
	  if(query[i] != '&')
	    {
	      pairs[j].value[k++] = query[i];
	    }
	  else
	    {
	      pairs[j].value[k] = '\0';
	      read_state = FINISHED;
	      k = 0;
	      j++;
	    }
	}
      i++;
    }

  /* Ako je upitni string zavrsen u toku citanja imena, moramo
     da zatvorimo string sa imenom, a vrednost je tada prazan
     string. */
  if(read_state == RD_NAME)
    {
      pairs[j].name[k] = '\0';
      pairs[j].value[0] = '\0';
      j++;
    }
  /* Ako je upitni string zavrsen u toku citanja vrednosti, tada
     je potrebno da zatvorimo string sa vrednoscu */
  else if(read_state == RD_VALUE)
    {
      pairs[j].value[k] = '\0';
      j++;
    }

  /* Vracamo broj parova u upitnom stringu */
  return j;
}

int main()
{
  query_pair pairs[100];

  /* Parsiramo upitni string */
  int n = parse_query_string(getenv("QUERY_STRING"), pairs);
  int i = 0;

  /* Obavezno je ispisati zaglavlje Content-Type. Ostala zaglavlja su
     opciona */
  printf("Content-Type: text/html\n");

  /* Ispisujemo praznu liniju */
  printf("\n");

  /* Ispisujemo html sadrzaj */
  printf("<!DOCTYPE html>\n");
  printf("<html>\n");
  printf("<head>\n");
  printf("<title>Odgovor servera</title>\n");
  printf("</head>\n");
  printf("<body>\n");
  printf("<h3>Vas upit:</h3>\n");
  printf("<ul>\n");

  /* Ispisujemo listu parova iz upitnog stringa */
  for(i = 0; i < n; i++)
    printf("<li>Ime: %s, Vrednost: %s</li>\n", pairs[i].name, pairs[i].value);
  printf("</ul>\n");

  printf("<h3>Promenljive okruzenja:</h3>\n");
  printf("<ul>\n");

  /* Ispisujemo listu promenljivih okruzenja */
  char ** p = environ;
  while(*p != NULL)
    {
      printf("<li>%s</li>\n", *p++);
    }
  
  printf("</ul>\n"); 
  printf("</body>\n");
  printf("</html>\n");
  
  return 0;
}
